@@ -18,6 +18,8 @@ from datetime import datetime
 
 from typing import Any, Dict, Optional, TypeVar, Union, cast
 
+import jsonschema.exceptions
+
 from sdsstools import get_logger, read_yaml_file
 from sdsstools.logger import SDSSLogger
 
@@ -246,6 +248,60 @@ class BaseClient(metaclass=abc.ABCMeta):
             "Sending commands is not implemented for this client."
         )
 
+    def proxy(self, actor: str) -> ProxyClient:
+        """Creates a proxy for an actor.
+
+        Returns a `.ProxyClient` that simplifies running a command multiple times.
+        For example ::
+
+            await client.send_command("focus_state", "moverelative -1000 UM")
+
+        can be replaced with ::
+
+            focus_stage = client.proxy("focus_stage")
+            await focus_stage.send_command("moverelative", "-1000", "UM")
+
+        """
+
+        return ProxyClient(self, actor)
+
+
+class ProxyClient:
+    """A proxy representing an actor.
+
+    Parameters
+    ----------
+    client
+        The client used to command the actor.
+    actor
+        The actor to command.
+
+    """
+
+    def __init__(self, client: BaseClient, actor: str):
+
+        self.client = client
+        self.actor = actor
+
+    def send_command(self, *args):
+        """Sends a command to the actor.
+
+        Returns the result of calling the client ``send_command()`` method
+        with the actor and concatenated arguments as parameters. Note that
+        in some cases the client ``send_command()`` method may be a coroutine
+        function, in which case the returned coroutine needs to be awaited.
+
+        Parameters
+        ----------
+        args
+            Arguments to pass to the actor. They will be concatenated using spaces.
+
+        """
+
+        command = " ".join(args)
+
+        return self.client.send_command(self.actor, command)
+
 
 class BaseActor(BaseClient):
     """An actor based on `asyncio`.
@@ -253,20 +309,39 @@ class BaseActor(BaseClient):
     This class expands `.BaseClient` with a parsing system for new commands
     and placeholders for methods for handling new commands and writing replies,
     which should be overridden by the specific actors.
+
+    Parameters
+    ----------
+    schema
+        The schema for the actor replies, as a JSONschema dictionary or path
+        to a JSON file. If `None`, defaults to the internal basic schema.
+    additional_properties
+        Whether to allow additional properties in the schema, other than the
+        ones defined by the schema. This parameter only is used if
+        ``schema=None`` or if ``additionalProperties`` is not defined in
+        the schema.
+
     """
 
     model: Union[Model, None] = None
 
-    def __init__(self, *args, schema: SchemaType = None, **kwargs):
+    def __init__(
+        self,
+        *args,
+        schema: SchemaType = None,
+        additional_properties: bool = False,
+        **kwargs,
+    ):
 
         super().__init__(*args, **kwargs)
 
-        self.load_schema(schema)
+        self.load_schema(schema, additional_properties=additional_properties)
 
     def load_schema(
         self,
         schema: Union[SchemaType, None],
         is_file=True,
+        additional_properties=False,
     ) -> Union[Model, None]:
         """Loads and validates the actor schema."""
 
@@ -275,11 +350,16 @@ class BaseActor(BaseClient):
                 "$schema": "http://json-schema.org/draft-07/schema#",
                 "type": "object",
                 "properties": {},
-                "additionalProperties": True,
+                "additionalProperties": additional_properties,
             }
             is_file = False
 
-        self.model = Model(self.name, schema, is_file=is_file, log=self.log)
+        self.model = Model(
+            self.name,
+            schema,
+            is_file=is_file,
+            additional_properties=additional_properties,
+        )
 
         return self.model
 
@@ -313,6 +393,7 @@ class BaseActor(BaseClient):
         command: Optional[BaseCommand] = None,
         broadcast: bool = False,
         validate: bool | None = None,
+        expand_exceptions: bool = True,
         silent: bool = False,
         call_internal: bool = True,
         **kwargs,
@@ -360,6 +441,11 @@ class BaseActor(BaseClient):
             Validate the reply against the actor schema. This is ignored if the actor
             was not started with knowledge of its own schema. If `None`, defaults to
             the actor global behaviour.
+        expand_exceptions
+            If the value of one of the keywords is an exception object and
+            `expand_exception=True`, the exception value will be converted into
+            a dictionary of ``exception_type`` and ``exception_message``. Otherwise
+            the exception will just be stringified.
         silent
             When `True` does not output the message to the users. This can be used to
             issue internal commands that update the internal model but that don't
@@ -378,10 +464,20 @@ class BaseActor(BaseClient):
         else:
             message = message or {}
             if not isinstance(message, dict):
-                raise TypeError("message must be a dictionary")
+                raise TypeError("message must be a string or a dictionary.")
 
         message.update(kwargs)
 
+        for key, value in message.items():
+            if isinstance(value, Exception):
+                if expand_exceptions is True:
+                    message[key] = {
+                        "exception_type": value.__class__.__name__,
+                        "exception_message": str(value),
+                    }
+                else:
+                    message[key] = str(value)
+
         reply = Reply(message_code, message, command=command, broadcast=broadcast)
 
         do_validate = validate if validate is not None else self.validate
@@ -389,7 +485,14 @@ class BaseActor(BaseClient):
             reply.use_validation = True
             result, err = self.model.update_model(message)
             if result is False:
-                message = {"error": f"Failed validating the reply: {err}"}
+                if isinstance(err, jsonschema.exceptions.ValidationError):
+                    message = {
+                        "error": f"Failed validating the reply: message {message} "
+                        "does not match the schema."
+                    }
+                else:
+                    message = {"error": f"Failed validating the reply: {err}"}
+
                 reply.message_code = "e"
                 reply.message = message
                 reply.validated = False
@@ -407,6 +510,13 @@ class BaseActor(BaseClient):
 
         return reply
 
+    def invoke_mock_command(self, command_str, command_id=0):
+        """Send a new command to an actor for testing.
+
+        Requires calling `.setup_test_actor`."""
+
+        raise NotImplementedError("setup_test_actor() has not been called.")
+
 
 class Reply:
     """A reply from a command or actor to be sent to the users."""
