@@ -77,18 +77,16 @@ class TronModel(BaseModel[TronKey]):
         A function or coroutine to call when the datamodel changes. The
         function is called with the instance of `.TronModel` and the modified keyword.
         If the callback is a coroutine, it is scheduled as a task.
-    log
-        Where to log messages.
+
     """
 
     def __init__(
         self,
         keydict: KeysDictionary,
         callback: Callable[[TronModel], Any] = None,
-        log: Optional[logging.Logger] = None,
     ):
 
-        super().__init__(keydict.name, callback=callback, log=log)
+        super().__init__(keydict.name, callback=callback)
 
         self.keydict = keydict
 
@@ -230,6 +228,8 @@ class TronConnection(BaseClient):
     def stop(self):
         """Closes the connection."""
 
+        assert self.transport
+
         self.transport.close()
 
     def connected(self):
@@ -242,6 +242,8 @@ class TronConnection(BaseClient):
 
     async def run_forever(self):  # pragma: no cover
 
+        assert self.transport
+
         # Keep alive until the connection is closed.
         while True:
             await asyncio.sleep(1)
@@ -289,6 +291,8 @@ class TronConnection(BaseClient):
 
         """
 
+        assert self.transport
+
         mid = mid or self._mid
 
         # The mid must be a 32-bit unsigned number.
